/*	l_dir.c	- Directory access utilityes
 *
 *	(C) by G. Caprino
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>			/* for strchr() */
#include <malloc.h>
#include <time.h>			/* for display file's date */

#include "readdir.h"

#ifdef WIN32
/* #define unlink __unlink			/* tc bug ? */
#include <io.h>
#else
#if defined(UNIX)
#include <sys/dir.h>
#include <ftw.h>
#else
#include <dirent.h>
#endif
#endif
#include <sys/stat.h>

extern	char	*getcwd();

struct	dskfile	*dskfiles, *lastdskfile;
int	ndskfiles;
int	ldir_getdirs;

static	char	filename[128];

static	void	*newMemory(int siz)
{
	void	*ptr;

	if(!(ptr = malloc(siz))) {
	    fprintf(stderr, "Out of memory!\n");
	    exit(1);
	}
	memset(ptr, 0, siz);
	return(ptr);
}

static	dircmp(a, b)
struct	dskfile **a, **b;
{
	return(strcmp(a[0]->name, b[0]->name));
}

#ifndef WIN32
typedef struct {
	int	fd;
} DIR;

static DIR	dirfd;

static struct	dirent {
	char	d_name[16];
} d;

static	DIR	*opendir(name)
char	*name;
{
	dirfd.fd = open(name, 0);
	return(dirfd.fd < 0 ? 0 : &dirfd);
}

static	struct	dirent *readdir(fd)
DIR	*fd;
{
	struct	direct	dd;

	while(read(fd->fd, &dd, sizeof(dd)) == sizeof(dd)) {
	  if(dd.d_ino == 0) continue;
	  strncpy(d.d_name, dd.d_name, DIRSIZ);
	  d.d_name[DIRSIZ] = 0;
	  return(&d);
	}
	return(0);
}

static	void	closedir(fd)
DIR	*fd;
{
	close(fd->fd);
}

#endif

void	freedskfiles()
{
	register struct dskfile *fe;

	if(ndskfiles)
	  for(fe = dskfiles; fe; fe = lastdskfile) {
	    lastdskfile = fe->next;
	    free(fe);
	  }
	ndskfiles = 0;
	dskfiles = 0;
}

#if 0
long unixdate(unsigned short date, unsigned short time)
{
	struct date d;
	struct time t;

	d.da_year = (date >> 9) + 1980;
	d.da_day = date & 0x1f;
	d.da_mon = (date >> 5) & 0xf;
	t.ti_sec = (time & 0x1f) * 2;
	t.ti_hund = 0;
	t.ti_min = (time >> 5) & 0x3f;
	t.ti_hour = (time >> 11) & 0x1f;
	return(dostounix(&d, &t));
}
#endif

void	get_directory(dirname, pattern)
char	*dirname, *pattern;
{
	register struct dskfile *fe;
	register int i;
	struct dskfile	**p;
	struct stat st;
#ifdef WIN32
	long	hndl;
	struct _finddata_t fileBlock;

	freedskfiles();
	hndl = _findfirst(pattern, &fileBlock);
	if(!hndl)
	    return;
	do {
	  if((fileBlock.attrib & _A_SUBDIR))
	    continue;			/* ignore unwanted file types */
	  fe = (struct dskfile *)newMemory(sizeof(*fe));
	  strcpy(fe->name, fileBlock.name);
	  fe->type = (fileBlock.attrib & 0x10) ? 1 : 0;
	  fe->size = fileBlock.size;
	  fe->date = fileBlock.time_write;
	  fe->next = 0;
	  if(!dskfiles)
	    dskfiles = fe;
	  else
	    lastdskfile->next = fe;
	  lastdskfile = fe;
	  ++ndskfiles;
	} while(_findnext(hndl, &fileBlock) >= 0);
	_findclose(&fileBlock);
#else /* __TURBOC__ */

#ifndef S_IFNAM
#define S_IFNAM	0		/* no named pipes on this system */
#endif
	DIR	*dir;
	struct dirent *entry;
	char	fullpath[256];

	freedskfiles();
	if(dir = opendir(dirname)) {
	  while(entry = readdir(dir)) {
	    strcpy(fullpath, dirname);
	    strcat(fullpath, "/");
	    strcat(fullpath, entry->d_name);
	    if(stat(fullpath, &st) < 0) continue;
	    if(!(st.st_mode & (S_IFDIR|S_IFREG))) continue;
	    fe = (struct dskfile *)newMemory(sizeof(*fe));
	    strcpy(fe->name, entry->d_name);
	    fe->type = (st.st_mode & (S_IFDIR|S_IFREG|S_IFCHR|S_IFNAM)) == S_IFDIR ? 1 : 0;
	    fe->size = st.st_size;
	    fe->date = st.st_mtime;
	    fe->next = 0;
	    if(!dskfiles)
	      dskfiles = fe;
	    else
	      lastdskfile->next = fe;
	    lastdskfile = fe;
	    ++ndskfiles;
	  }
	  closedir(dir);
	}
#endif
	if(ndskfiles) {
	  p = (struct dskfile **)newMemory(sizeof(struct dskfile *) * ndskfiles);
	  for(i = 0, fe = dskfiles; fe && i < ndskfiles; ++i, fe = fe->next)
	    p[i] = fe;
	  qsort(p, ndskfiles, sizeof(*p), dircmp);
	  dskfiles = lastdskfile = 0;
	  for(i = 0; i < ndskfiles; ++i) {
	    p[i]->next = 0;
	    if(!dskfiles)
	      dskfiles = p[i];
	    else
	      lastdskfile->next = p[i];
	    lastdskfile = p[i];
	  }
	  free(p);
	} else
	  dskfiles = lastdskfile = 0;
}

