/*	menu.c - Created by Giampiero Caprino

This program is part of Train Director.

Train Director is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Train Director is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Train Director; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.

*/

/*	This program is used to process train timetables
 *	as returned by the web site http://reiseauskunft.bahn.de
 *	or http://bahn.hafas.de/bin/bhftafel.exe/en
 *
 *	wmatch *.htm
 *		extracts info from the arrival/departure
 *		page of a single station.
 *	wmatch -l file
 *		creates a shell script with the list
 *		of trains arriving/departing from a station
 *	... then run the shell script to get the info for
 *		each individual train.
 *	wmatch -t *.tr
 *		creates a timetable file for use by Train Director.
 *		The file must then be edited to insert
 *		entry and exit point.
 *	wmatch -a file.htm
 *		creates a Train Director schedule file (.sch)
 *		by requesting the info for all the pages and all
 *		the trains that arrive/depart from the station
 *		to which the file.htm refers to.
 */

#if 0
#define	DBHOME "http://reiseauskunft.bahn.de"
#else
#define DBHOME "http://bahn.hafas.de"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <memory.h>
#include "readdir.h"

struct	statn {
	struct statn *next;
	char	*name;
	char	*ref;
} *stations, *trlist;

struct	ttbl {
	struct ttbl *next;
	char	*ref;
	char	*source;
} *ttbls;

char	*text;
int	curtxt, maxtxt;
char	buff[2048];
char	train[256];
char	dest[256];
char	stop[256];
char	hour[256];
char	dep[256];
int	isall;				/* -a option: get all info in one run */
char	*debugtxt;

#ifdef WIN32
#define	sleep wsleep
void	wsleep(int ns)
{
	long	t;

	t = time(NULL);
	while(time(NULL) == t);
}
#endif

#ifdef FROM_WINDOW

#include <stdarg.h>

void	win_printf(char *fmt, ...)
{
	va_list	arg;
	char	buff[1024];

	va_start(arg, fmt);
	vsprintf(buff, fmt, arg);
	va_end(arg);
	win_trace(buff);
}

void	win_fprintf(FILE *fp, char *fmt, ...)
{
	va_list	arg;
	char	buff[1024];

	va_start(arg, fmt);
	if(fp != stderr)
	    vfprintf(fp, fmt, arg);
	else {
	    vsprintf(buff, fmt, arg);
	    win_trace(buff);
	}
	va_end(arg);
}

#define	printf	win_printf
#define	fprintf	win_fprintf

#endif

int	fgetline(char *buff, int size, FILE *fp)
{
	int	l;

	if(!fgets(buff, size, fp))
	    return 0;

	buff[size - 1] = 0;
	l = strlen(buff);
	if(l > 0 && buff[l - 1] == '\n') --l;
	if(l > 0 && buff[l - 1] == '\r') --l;
	if(l > 0 && buff[l - 1] == '\r') --l;
	buff[l] = 0;
	return 1;
}

int	get_file(FILE *fi)
{
	int	l;

	maxtxt = 1000;
	text = (char *)malloc(maxtxt);
	curtxt = 0;
	while(fgetline(buff, sizeof(buff), fi)) {
	    l = strlen(buff);
	    if(curtxt + l + 2 > maxtxt) {
		maxtxt = curtxt + l + 1000;
		text = (char *)realloc(text, maxtxt);
		if(!text) {
		    printf("Not enough memory for page of %d bytes\n", maxtxt);
		    return 0;
		}
	    }
	    memcpy(text + curtxt, buff, l + 1);
	    curtxt += l + 1;
	}
	text[curtxt] = 0x7f;			/* EOF */
	return 1;
}

char	*find_match0(char *p, char *str)
{
	int	l;

	if(!p)
	    return 0;
	debugtxt = p;
	l = strlen(str);
	while(strnicmp(p, str, l)) {
	    while(*p != 0x7f && *p) ++p;	/* go to next line */
	    if(*p) 				/* reached end of file */
		return 0;
	    ++p;
 	    while(*p == ' ') ++p;
	}
	return p + l;
}

char	*find_match(char *p, char *str)
{
	int	l;

	if(!p || *p == 0x7f)
	    return 0;
	debugtxt = p;
	l = strlen(str);
	while(strnicmp(p, str, l)) {
	    if(*++p == 0x7f) 			/* reached end of file */
		return 0;
	}
	return p + l;
}

void	remove_html(char *p)
{
	char	*d;

	if(!strnicmp(p, "<A ", 3)) {
	    d = strchr(p, '>');
	    if(!d)
		return;
	    while(*p++ = *++d);
	}
}

void	remove_spaces(char *p)
{
	char	*d;

	for(d = p; *p; ++p) {
	    if(*p == ' ')
		continue;
	    *d++ = *p;
	}
	*d = 0;
}

char	*parse_hour(char *p)
{
	int	i;

	if(!p || *p == 0x7f)
	    return 0;
	i = 0;
	while(*p == ' ') ++p;
	while(*p >= '0' && *p <= '9')
	    hour[i++] = *p++;
	if(*p == ':')
	    hour[i++] = *p++;
	while(*p >= '0' && *p <= '9')
	    hour[i++] = *p++;
	hour[i] = 0;
	return p;
}

struct statn *find_name(struct statn *tp, char *name)
{
	while(tp) {
	    if(!strcmp(tp->name, name))
		return tp;
	    tp = tp->next;
	}
	return 0;
}

char	*parse_train(char *p)
{
	int	i;
	char	*p1;
	char	buff[256];
	char	buff1[256];
	struct statn *tp;
 
	if(!p || *p == 0x7f)
	    return 0;
	i = 0;
	while(*p != 0x7f) {
	    if(!strncmp(p, "&nbsp;", 6))
		p += 6;
	    else if(!*p || *p == ' ')
		++p;
	    else
		break;
	}
	while(*p != 0x7f && strnicmp(p, "</A>", 4) && i < sizeof(train) - 1)
	    if(*p)
		train[i++] = *p++;
	    else
		++p;
	train[i] = 0;
	if(*p == 0x7f)
	    return 0;
	if(!strnicmp(train, "<A ", 3) && (p1 = strchr(train, '"'))) {
	    for(i = 0; *++p1 != '"'; ++i)	/* url */
		buff1[i] = *p1;
	    buff1[i] = 0;
	    remove_html(train);
	    strcpy(buff, train);
	    remove_spaces(buff);
	    if(isall) {
		if(!find_name(trlist, buff)) {
		    tp = (struct statn *)calloc(sizeof(struct statn), 1);
		    tp->name = strdup(buff);
		    tp->ref = strdup(buff1);
		    tp->next = trlist;
		    trlist = tp;
		}
	    } else
		printf("#'%s %s\n", buff, buff1);
	    fflush(stdout);
	} else
	    remove_html(train);
	return p + 4;
}

char	*parse_dest(char *dst, char *p)
{
	char	*tp;

	tp = dst;
	if(!p || *p == 0x7f)
	    return 0;
	while(!*p || *p == ' ') ++p;
	while(*p != 0x7f && strnicmp(p, "</A>", 4) && strnicmp(p, "</B>", 4))
	    if(*p)
		*dst++ = *p++;
	    else
		++p;
	*dst = 0;
	remove_html(tp);
	if(*p == 0x7f)
	    return 0;
	return p + 4;
}

struct statn	*record_station(char *p)
{
	struct	statn *st;

	for(st = stations; st; st = st->next) {
	    if(!strcmp(p, st->name))
		return st;
	}
	st = (struct statn *)malloc(sizeof(struct statn));
	memset(st, 0, sizeof(struct statn));
	st->name = strdup(p);
	st->next = stations;
	stations = st;
	return(st);
}

int	parse_station_file(FILE *fi)
{
	char	*p;

	if(!get_file(fi))
	    return 0;
	if(find_match(text, "No trains in this space of time."))
	    return 0;
	if(!(p = find_match(text, "Platform")))
	    return 0;
agn:
	p = find_match0(p, "<TD");
	if(!p)
	    return 0;
	p = find_match(p, "<B>");
	p = parse_hour(p);
	strcpy(dep, hour);
	p = find_match(p, "<B>");
	p = parse_train(p);
	p = find_match(p, "<B>");
	p = parse_dest(dest, p);
	p = find_match(p, "<BR>");
	if(!p)
	    return 0;
	if(!isall) {
	    printf("Train: %s\n", train);
	    printf("\tHour: %s\n", dep);
	    printf("\tFrom/to: %s\n", dest);
	}
	record_station(dest);
	while(strnicmp(p, "<BR>", 4)) {
	    while(*p == ' ') ++p;
	    if(!*p) {
		++p;
		continue;
	    }
	    if(!strnicmp(p, "</FONT>", 7)) {
		p += 7;
		continue;
	    }
	    if(!strnicmp(p, "</TD>", 5)) {
		p += 5;
		break;
	    }
	    p = parse_dest(stop, p);
	    p = parse_hour(p);
	    if(!isall)
		printf("\t\t%s %s\n", hour, stop);
	    if(!strncmp(p, " - ", 3))
		p += 3;
	    else if(!strcmp(p, " -"))
		p += 2;
	    if(!strnicmp(p, "&nbsp;", 6))
		p += 6;
	    if(!strnicmp(p, "<IMAGE ", 7)) {
		while(*p != 0x7f && strnicmp(p, "&nbsp;", 6)) ++p;
		if(*p != 0x7f)
		    p += 6;
	    }
	}
	if(!isall)
	    printf(".\n\n");
	p = find_match0(p, "</TR>");
	goto agn;
}

char	*hour_column(char *p, char *hour)
{
	if(!(p = find_match(p, "ALIGN=")))
	    return p;
	if(!strncmp(p, "RIGHT", 5))
	    p += 5;
	else if(!strncmp(p, "\"RIGHT\"", 7))
	    p += 7;
	if(*p == '>')
	    ++p;
	else
	    p = find_match(p, " CLASS=\"result\">");
	strcpy(hour, "0:00");
	while(p && *p != 0x7f) {
	    if(*p == ' ' || !*p) ++p;
	    else if(!strnicmp(p, "&nbsp;", 6))
		p += 6;
	    else
		break;
	}
	if(p && strnicmp(p, "&nbsp;", 6)) {	/* no departure! */
/*	    p = find_match(p, "<B>"); */
	    p = parse_hour(p);
	}
	return p;
}

int	parse_train_info(FILE *fi)
{
	char	*p, *p1;
	char	arr[64];
	static	char	*days[]  = { "Mo", "Tu", "We", "Th", "Fr", "Sa", "Su", 0 };
	int	i;
	int	lastday;
	int	firsttime;

	if(!get_file(fi))
	    return 0;
	p = find_match(text, "Station");
	p = find_match(p, "Train");
/*	p = find_match(p, "COLSPAN=");
	p = find_match(p, "<B>");*/
	p = find_match0(p, "<A ");
	if(!p)
	    return 0;
	p -= 3;
	p = parse_dest(stop, p);
	p = find_match0(p, "<TD ");	/* skip arrival at originating st. */
	p = find_match0(p, "<TD ");	/* go to depart. at originating st. */
	p = hour_column(p, hour);
	p = find_match0(p, "<TD ");	/* go to train name */
/*	p = find_match(p, "<B>");
	p1 = find_match(p, "</B>"); */
	if(!p)
	    return 0;
	while(*p = 0x7f && *p) ++p;
	if(!*p) ++p;
	for(p1 = p; *p1 != 0x7f && *p1; ++p1);
	if(*p1)
	    return 0;
	if(p1 - p >= sizeof(train)) {
	    printf("##Can't find train name!\n");
	    return 0;
	}
	strncpy(train, p, p1 - p);
	train[p1 - p] = 0;
	if((p1 = strchr(train, '<')))
	    *p1 = 0;
	remove_spaces(train);
	printf("Train: %s\n\tEnter: %s, %s\n", train, hour, stop);
	for(;;) {
	    p1 = p;
/*	    p = find_match(p, "COLSPAN");
	    p = find_match(p, "<B>"); */
	    if((p = find_match0(p, "<A "))) {
		p -= 3;
		goto do_station;
	    }
	    if(!p || !*p) {
		if(!(p = find_match(p1, "Days of operation:")))
		    break;
		if(!(p = find_match(p, "COLSPAN")))
		    break;
/*		p = find_match(p, "<B>"); */
		while(*p != 0x7f && *p != '>') ++p;
		if(*p == '>') ++p;
		if(!*p) ++p;
		if(strnicmp(p, " runs ", 6))
		    break;
		p += 6;
		if(strstr(p, "daily"))
		    break;
		firsttime = 1;
agn:		for(i = 0; days[i]; ++i) {
		    if(!(p1 = strstr(p, days[i])))
			continue;
		    if(!strnicmp(p1 + 2, " - ", 3)) {
			lastday = i;
			for(++i; days[i]; ++i) {
			    if(!strnicmp(p1 + 5, days[i], 2)) {
				if(firsttime)
				    printf("When: ");
				while(lastday <= i)
				    printf("%d", ++lastday);
				p = p1 + 7;
				firsttime = 0;
				i = -1;
				goto agn;
			    }
			}
		    }
		    if(firsttime)
			printf("When: ");
		    printf("%d", i + 1);
		    if(p1[2] == ';')
			break;
		    if(p1[2] != ',')
			break;
		    p = p1 + 3;
		    i = -1;
		    firsttime = 0;
		}
		printf("\n");
		break;
	    }
do_station:
	    p = parse_dest(dest, p);
/*	    p = find_match(p, "ALIGN=RIGHT");
	    if(!p)
		break; */
	    if(!(p = find_match(p, "<TD ")))
		break;
	    while(*p != 0x7f && *p != '>') ++p;
	    if(*p == '>') ++p;
	    if(!*p) ++p;
	    if(strnicmp(p, "&nbsp;", 6)) {
/* 		p = find_match(p, "<B>"); */
		p = parse_hour(p);
		strncpy(arr, hour, sizeof(arr) - 1);
		arr[sizeof(arr) - 1] = 0;
/*		p = find_match(p, "ALIGN=RIGHT");
		if(!p)
		    continue; */
		if(!(p = find_match(p, "<TD ")))
		    continue;
		while(*p != 0x7f && *p != '>') ++p;
		if(*p == '>') ++p;
		if(!*p) ++p;
		if(!strnicmp(p, "&nbsp;", 6) ||
		    !strnicmp(p, "</TD>", 5)) {/* arrival st. has no departure time */
		    printf("\t\t%s, -, %s\n", arr, dest);
		    continue;
		}
/* 		p = find_match(p, "<B>");*/
		p = parse_hour(p);
		printf("\t\t%s, %s, %s\n", arr, hour, dest);
	    } else {
		p = find_match(p, "ALIGN=RIGHT>");
		if(!p)
		    break;
		if(!strnicmp(p, "&nbsp;", 6))/* no arrival, no departure-ignore*/
		    continue;
		p = find_match(p, "<B>");
		p = parse_hour(p);
		printf("\t\t%s, %s, %s\n", hour, hour, dest);
	    }
	}
done:
	printf(".\n\n");
}

void	dump_stations(void)
{
	struct statn *st;

	for(st = stations; st; st = st->next)
	    printf("#<%s\n", st->name);
}

void	parse_train_list(FILE *fp)
{
	char	buff[256];
	int	l;
	char	*p;
	struct	statn *sp;

	while(fgetline(buff, sizeof(buff), fp)) {
	    l = strlen(buff);
	    if(!l)
		continue;
	    if(buff[0] != '#' || buff[1] != '\'')
		continue;
	    if(!(p = strchr(buff, ' ')))
		continue;
	    *p = 0;
	    while(*++p == ' ');
	    sp = record_station(buff + 2);
	    if(!sp->ref)
		sp->ref = strdup(p);
	}
	for(sp = stations; sp; sp = sp->next) {
	    printf("wget %s%s > %s.tr\n", DBHOME, sp->ref, sp->name);
/*	    printf("wget http://bahn.hafas.de/bin/bhftafel.exe/en%s > %s.tr\n", sp->ref, sp->name);*/
	}
}

char	*get_hour_head(char *p, char hours[24])
{
	int	h;

	for(h = 0; *p >= '0' && *p <= '9'; ++p)
	    h = h * 10 + *p - '0';
	if(*p == ':')
	    while(*++p >= '0' && *p <= '9');
	hours[h % 24] = 1;
	return p;
}

void	add_ttable(char *buff, char *source)
{
	struct ttbl *tp;

	tp = (struct ttbl *)calloc(sizeof(struct ttbl), 1);
	tp->ref = strdup(buff);
	tp->source = strdup(source);
	tp->next = ttbls;
	ttbls = tp;
}

int	parse_all(char *fname, FILE *fp)
{
	char	*p;
	char	hours[24];
	char	*href, *hrefarr, *hreftime;
	int	i;
	int	no_ab;
	char	buff[512];
	struct	ttbl *tp;

	if(!get_file(fp))
	    return 0;
	memset(hours, 0, sizeof(hours));
	href = 0;

	/* locate list of time pages */

	p = find_match0(text, "<TD COLSPAN=\"4\" BGCOLOR=\"#DDDDDD\"");
	if(!p)
	    p = find_match0(text, "<td bgcolor=#DDDDDD colspan=4");
	if(!p) {
	    if(!(p = find_match0(text, "<TD BGCOLOR=\"#BBBBBB\"")) &&
		!(p = find_match0(text, "<TD BGCOLOR=#BBBBBB")))
		return 0;
	    if(!(fp = fopen(fname, "r"))) {
		perror(fname);
		return 0;
	    }
	    parse_station_file(fp);	/* builds trlist */
	    fclose(fp);
	    return 1;
	}
	while(*p && *p != 0x7f) ++p;
	if(!*p) ++p;
	while(p && *p != 0x7f) {
	    while(*p == ' ') ++p;
	    if(!strnicmp(p, "<B>", 3)) {/* the current page */
		p = get_hour_head(p + 3, hours);
	    } else if(!strnicmp(p, "<A HREF=\"", 9)) { /* another page */
		p += 9;
scan_link:
		for(i = 0; i < sizeof(buff) && *p != '"'; ++i)
		    buff[i] = *p++;
		buff[i] = 0;
		if(!href)
		    href = strdup(buff);
		while(*p != 0x7f && *p != '>') ++p;
		if(*p == '>') ++p;
		p = get_hour_head(p, hours);
	    } else if(!strnicmp(p, "size=2>", 7)) {
		p += 7;
		continue;
	    } else if(!strnicmp(p, "<A ", 3)) {
		p += 3;
		if(!*p) ++p;
		continue;
	    } else if(!strnicmp(p, "href=\"", 6)) {
		p += 6;
		goto scan_link;
	    } else if(strnicmp(p, "<BR>", 4))
		break;
	    while(*p && *p != 0x7f) ++p;
	    if(!*p) ++p;
	}
	if(!href) {
	    if(!(p = find_match(text, "00:00-23:59</B>"))) {
		printf("Error: no link to time pages!\n");
		return 0;
	    }
	    fprintf(stderr, "Warning: single page; you must provide the arrival/departure page as well.\n");
	    rewind(fp);
	    parse_station_file(fp);
	    return 1;
	}
#if 0
	hrefarr = strstr(href, "AnAb=");
	hreftime = strstr(href, "start=");
	if(!hrefarr || !hreftime) {
	    printf("Error: no reference to Arr/Dep or time in URL!\n");
	    return 0;
	}
	*hrefarr = 0;
	*hreftime = 0;
	for(i = 0; i < 24; ++i)
	    if(hours[i]) {
		sprintf(buff, "%sAnAb=AN%sstart=%02d:00%s",
			href, hrefarr + 7, i, hreftime + 11);
		add_ttable(buff, fname);
		sprintf(buff, "%sAnAb=AB%sstart=%02d:00%s",
			href, hrefarr + 7, i, hreftime + 11);
		add_ttable(buff, fname);
	    }
#else
	hrefarr = strstr(href, "/an?proto");
	if(!hrefarr)
	    hrefarr = strstr(href, "/an?P");
	if(!hrefarr)
	    hrefarr = strstr(href, "/ab?proto");
	if(!hrefarr)
	    hrefarr = strstr(href, "/ab?P");
	no_ab = 0;
	if(!hrefarr) {
	    hrefarr = strstr(href, "?P=");
	    if(hrefarr)
		no_ab = 1;
	}
	if(!hrefarr) {
err:	    printf("Error: no reference to Arr/Dep or time in URL!\n");
	    return 0;
	}
	for(hreftime = hrefarr - 1; hreftime > href && *hreftime != '/'; --hreftime);
	if(hreftime == href)
	    goto err;
	*hreftime = 0;
	for(i = 0; i < 24; ++i) {
	    if(!hours[i])
		continue;
	    sprintf(buff, "%s/%02d:00/an%s", href, i, hrefarr + 3);
	    add_ttable(buff, fname);
	    if(no_ab)
		sprintf(buff, "%s/%02d:00/%s", href, i, hrefarr + 3);
	    else
		sprintf(buff, "%s/%02d:00/ab%s", href, i, hrefarr + 3);
	    add_ttable(buff, fname);
	}
#endif
	return 1;
}

int	trsort(struct statn **a, struct statn **b)
{
	return(-strcmp((*a)->name, (*b)->name));
}

void	sort_train_list(void)
{
	int	ntrains;
	struct	statn	*tr, **tarray;
	int	i;

	for(ntrains = 0, tr = trlist; tr; tr = tr->next)
	    ++ntrains;
	if(!ntrains)
	    return;
	tarray = (struct statn **)malloc(sizeof(struct statn *) * ntrains);
	for(i = 0, tr = trlist; tr; tr = tr->next, ++i)
	    tarray[i] = tr;
	qsort(tarray, ntrains, sizeof(struct statn *), trsort);
	
	trlist = 0;
	for(i = 0; i < ntrains; ++i) {
	    tr = tarray[i];
	    tr->next = trlist;
	    trlist = tr;
	}
	free(tarray);
}

void	get_all_trains(void)
{
	FILE	*fp;
	struct statn *tr;
	char	cmd[256];
	char	buff[256];

	sort_train_list();
	for(tr = trlist; tr; tr = tr->next) {
	    sprintf(buff, "%s.tr", tr->name);
	    if(!(fp = fopen(buff, "w"))) {
		perror(buff);
		continue;
	    }
	    sprintf(cmd, "%s%s", DBHOME, tr->ref);
	    fprintf(stderr, "# Reading %s\n", buff);
	    if(!wget(fp, cmd))
		return;
	    fclose(fp);
	    fprintf(stderr, "# Read %s\n", buff);
	    sleep(1);
	    if(!(fp = fopen(buff, "r"))) {
		perror(buff);
		continue;
	    }
	    parse_train_info(fp);
	    fclose(fp);
	}
}

void	get_all_ttbls(void)
{
	struct	ttbl *tp;
	struct	statn *tr;
	char	buff[32];
	char	cmd[256];
	char	*lastsrc;
	int	i;
	FILE	*fp;

	if(!getenv("PROXY"))
	    fprintf(stderr, "WARNING: PROXY env. variable not set.\n");
	i = 0;
	lastsrc = 0;
	for(tp = ttbls; tp; tp = tp->next) {
	    if(!lastsrc || strcmp(lastsrc, tp->source)) {
		lastsrc = tp->source;
		fprintf(stderr, "# Processing %s\n", lastsrc);
	    }
	    sprintf(buff, "page%02d", ++i);
	    if(!(fp = fopen(buff, "w"))) {
		fprintf(stderr, "Can't create '%s'\n", buff);
		return;
	    }
	    sprintf(cmd, "%s/%s", DBHOME, tp->ref); 
/*	    sprintf(cmd, "http://bahn.hafas.de/bin/bhftafel.exe/en/%s", tp->ref);*/
	    fprintf(stderr, "# Reading %s\n", buff);
	    if(!wget(fp, cmd))
		return;
	    fclose(fp);
	    sleep(1);

	    if(!(fp = fopen(buff, "r"))) {
		perror(buff);
		continue;
	    }
	    parse_station_file(fp);	/* builds trlist */
	    fclose(fp);
	}
	get_all_trains();
}

int	gentt_main(int argc, char *argv[])
{
	FILE	*fi;
	int	l;
	int	istrain, istlist;
	struct dskfile *df;

	if(argc < 2)
	    return parse_station_file(stdin);
	istrain = 0;
	istlist = 0;
	isall = 0;
	if(!strcmp(argv[1], "-a")) {
	    isall = 1;
	    argv[1] = argv[0];
	    ++argv;
	    --argc;
	    if(!socket_init())
		exit(1);
	} else if(!strcmp(argv[1], "-t")) {
	    istrain = 1;
	    argv[1] = argv[0];
	    ++argv;
	    --argc;
	} else if(!strcmp(argv[1], "-l")) {
	    istlist = 1;
	    argv[1] = argv[0];
	    ++argv;
	    --argc;
	}
	printf("#!trdir\n");
	for(l = 1; l < argc; ++l) {
	  get_directory(".", argv[l]);
	  for(df = dskfiles; df; df = df->next) {
	    if(!(fi = fopen(df->name, "r"))) {
		perror(df->name);
		continue;
	    }
	    if(istrain)
		parse_train_info(fi);
	    else if(istlist)
		parse_train_list(fi);
	    else if(isall)
		parse_all(df->name, fi);
	    else
		parse_station_file(fi);
	    fclose(fi);
	  }
	}
	if(isall)
	    get_all_ttbls();
	else if(!istlist && !istrain)
	    dump_stations();
	return 0;
}

#ifndef FROM_WINDOW
int	main(int argc, char *argv[])
{
	return gentt_main(argc, argv);
}
#endif