#include <stdio.h>
#include <string.h>
#include <sys/types.h>		/* general system types */
#include <signal.h>		/* for signals */
#ifdef unix
#include <sys/time.h>		/* for select and the like */
#include <sys/socket.h>		/* socket calls */
#include <sys/un.h>		/* unix socket calls */
#include <netinet/in.h>		/* internet socket calls */
#include <netdb.h>		/* host to addr calls */
struct hostent *gethostbyname();
#else
#include <winsock.h>
#endif

#include "sock.h"

static	int	addr_fam, form_type;
static	struct sockaddr_in sock_name;
static	char	hostname[256];
static	struct sockaddr *targ;
static	int	targ_len;

int socket_init(void)
{
#ifndef unix
	int len;
	WSADATA WSAdata;
	WORD wVersionRequested = MAKEWORD(1, 1);

	memset(&WSAdata, 0, sizeof(WSAdata));
	len = WSAStartup(wVersionRequested, &WSAdata);
	if(len) {
	    fprintf(stderr, "WSAStartup failed: code 0x%x\n", len);
	    return 0;
	}
#endif
	return 1;
}

conn	*socket_connect(char *host, int port)
{
	int len;
	int sock;
	struct hostent *hp;
	conn	*cp;

	addr_fam = AF_INET;	/* internet address family */
	form_type = SOCK_STREAM;/* stream format (TCP or pipe) */

	sock_name.sin_port = htons(port);
	if(!host) {
	    gethostname(hostname, sizeof(hostname));
	    if (!(hp = gethostbyname(hostname))) {
		fprintf(stderr, "cannot find host '%s' in database", hostname);
		return(0);
	    }
	    host = hostname;
	    memcpy((char *)&sock_name.sin_addr, hp->h_addr, hp->h_length);
	} else if (!isalpha(host[1])) {	/* addr of connection */
	    char	cbuf[4];
	    int	i0, i1, i2, i3;

	    sscanf(host, "%d.%d.%d.%d", &i0, &i1, &i2, &i3);
	    cbuf[0] = i0;
	    cbuf[1] = i1;
	    cbuf[2] = i2;
	    cbuf[3] = i3;
	    memcpy((char *)&sock_name.sin_addr, cbuf, 4); /* copy in number */
	} else {			/* hostname */
	    if (!(hp = gethostbyname(host))) {
		fprintf(stderr, "cannot find host '%s' in database", host);
		return(0);
	    }
	    memcpy(&sock_name.sin_addr, hp->h_addr, hp->h_length);
	}
	sock_name.sin_family = /*htons*/(AF_INET);
	targ = (struct sockaddr *)&sock_name;
	targ_len = sizeof(sock_name);

	if ((sock = socket(addr_fam, form_type, 0)) < 0) {
	    perror("socket");
	    return(0);
	}
	if((len = connect(sock, targ, targ_len))) {
	    perror("connect");
	    fprintf(stderr, "Cannot connect to %s:%d\n", host, port);
	    return(0);
	}
	if(!(cp = (conn *)calloc(sizeof(conn), 1))) {
	    fprintf(stderr, "No memory for server structure\n");
	    return 0;
	}
	cp->sock = sock;
	return(cp);
}

int sock_rdline(conn *cp, char *dst)
{
	int	len;

	len = 0;
agn:
	if(cp->inidx == cp->inlim) {
	    cp->inlim = recv(cp->sock, cp->iline, sizeof(cp->iline), 0);
	    if(!cp->inlim)
		return(0);
	    if(cp->inlim < 0) {
		perror("read");
		cp->inlim = cp->inidx;
		goto agn;
	    }
	    cp->inidx = 0;
	}
	for(; cp->inidx < cp->inlim; ++len) {
	    dst[len] = cp->iline[cp->inidx];
	    if(cp->iline[cp->inidx++] == '\n') {
		dst[++len] = 0;
		return(len);
	    }
	}
	goto agn;
}

void	sock_nocrlf(char *str)
{
	int	l;

	l = strlen(str);
	if(l && str[l - 1] == '\n') --l;
	if(l && str[l - 1] == '\r') --l;
	str[l] = 0;
}

int	sock_rdfile(conn *cp, FILE *fp, int nlines)
{
	char	buff[2048];

	while(nlines > 0 && sock_rdline(cp, buff)) {
	    fprintf(fp, "%s\n", buff);
	    --nlines;
	}
	return nlines == 0;
}

void	sock_sendcmd(conn *cp, char *str)
{
	send(cp->sock, str, strlen(str), 0);
}
